API — AsyncClick Documentation (8.x)

# API

This part of the documentation lists the full API reference of all public classes and functions.

## Decorators

`asyncclick.` `command`(name=None, cls=None, **attrs)

  Creates a new `Command` and uses the decorated function as callback. This will also automatically attach all decorated `option()`s and `argument()`s as parameters to the command.

  The name of the command defaults to the name of the function with underscores replaced by dashes. If you want to change that, you can pass the intended name as the first argument.

  All keyword arguments are forwarded to the underlying command class.

  Once decorated the function turns into a `Command` instance that can be invoked as a command line utility or be attached to a command `Group`.

  Parameters

    - name – the name of the command. This defaults to the function name with underscores replaced by dashes.

    - cls – the command class to instantiate. This defaults to `Command`.

`asyncclick.` `group`(name=None, **attrs)

  Creates a new `Group` with a function as callback. This works otherwise the same as `command()` just that the cls parameter is set to `Group`.

`asyncclick.` `argument`(*param_decls, **attrs)

  Attaches an argument to the command. All positional arguments are passed as parameter declarations to `Argument`; all keyword arguments are forwarded unchanged (except `cls`). This is equivalent to creating an `Argument` instance manually and attaching it to the `Command.params` list.

  Parameters

    cls – the argument class to instantiate. This defaults to `Argument`.

`asyncclick.` `option`(*param_decls, **attrs)

  Attaches an option to the command. All positional arguments are passed as parameter declarations to `Option`; all keyword arguments are forwarded unchanged (except `cls`). This is equivalent to creating an `Option` instance manually and attaching it to the `Command.params` list.

  Parameters

    cls – the option class to instantiate. This defaults to `Option`.

`asyncclick.` `password_option`(*param_decls, **kwargs)

  Add a `--password` option which prompts for a password, hiding input and asking to enter the value again for confirmation.

  Parameters

    - param_decls – One or more option names. Defaults to the single value `"--password"`.

    - kwargs – Extra arguments are passed to `option()`.

`asyncclick.` `confirmation_option`(*param_decls, **kwargs)

  Add a `--yes` option which shows a prompt before continuing if not passed. If the prompt is declined, the program will exit.

  Parameters

    - param_decls – One or more option names. Defaults to the single value `"--yes"`.

    - kwargs – Extra arguments are passed to `option()`.

`asyncclick.` `version_option`(version=None, *param_decls, package_name=None, prog_name=None, message='%(prog)s, version %(version)s', **kwargs)

  Add a `--version` option which immediately prints the version number and exits the program.

  If `version` is not provided, Click will try to detect it using `importlib.metadata.version()` to get the version for the `package_name`. On Python < 3.8, the `importlib_metadata` backport must be installed.

  If `package_name` is not provided, Click will try to detect it by inspecting the stack frames. This will be used to detect the version, so it must match the name of the installed package.

  Parameters

    - version – The version number to show. If not provided, Click will try to detect it.

    - param_decls – One or more option names. Defaults to the single value `"--version"`.

    - package_name – The package name to detect the version from. If not provided, Click will try to detect it.

    - prog_name – The name of the CLI to show in the message. If not provided, it will be detected from the command.

    - message – The message to show. The values `%(prog)s`, `%(package)s`, and `%(version)s` are available.

    - kwargs – Extra arguments are passed to `option()`.

  Raises

    RuntimeError – `version` could not be detected.

  Changed in version 8.0: Add the `package_name` parameter, and the `%(package)s` value for messages.

  Changed in version 8.0: Use `importlib.metadata` instead of `pkg_resources`.

`asyncclick.` `help_option`(*param_decls, **kwargs)

  Add a `--help` option which immediately prints the help page and exits the program.

  This is usually unnecessary, as the `--help` option is added to each command automatically unless `add_help_option=False` is passed.

  Parameters

    - param_decls – One or more option names. Defaults to the single value `"--help"`.

    - kwargs – Extra arguments are passed to `option()`.

`asyncclick.` `pass_context`(f)

  Marks a callback as wanting to receive the current context object as first argument.

`asyncclick.` `pass_obj`(f)

  Similar to `pass_context()`, but only pass the object on the context onwards (`Context.obj`). This is useful if that object represents the state of a nested system.

`asyncclick.` `make_pass_decorator`(object_type, ensure=False)

  Given an object type this creates a decorator that will work similar to `pass_obj()` but instead of passing the object of the current context, it will find the innermost context of type `object_type()`.

  This generates a decorator that works roughly like this:

      from functools import update_wrapper

      def decorator(f):
          @pass_context
          def new_func(ctx, *args, **kwargs):
              obj = ctx.find_object(object_type)
              return ctx.invoke(f, obj, *args, **kwargs)
          return update_wrapper(new_func, f)
      return decorator

  Parameters

    - object_type – the type of the object to pass.

    - ensure – if set to True, a new object will be created and remembered on the context if it’s not there yet.

## Utilities

`asyncclick.` `echo`(message=None, file=None, nl=True, err=False, color=None)

  Prints a message plus a newline to the given file or stdout. On first sight, this looks like the print function, but it has improved support for handling Unicode and binary data that does not fail no matter how badly configured the system is.

  Primarily it means that you can print binary data as well as Unicode data on both 2.x and 3.x to the given file in the most appropriate way possible. This is a very carefree function in that it will try its best to not fail. As of Click 6.0 this includes support for unicode output on the Windows console.

  In addition to that, if colorama is installed, the echo function will also support clever handling of ANSI codes. Essentially it will then do the following:

  - add transparent handling of ANSI color codes on Windows.

  - hide ANSI codes automatically if the destination file is not a terminal.

  Changelog

  Changed in version 6.0: As of Click 6.0 the echo function will properly support unicode output on the windows console. Not that click does not modify the interpreter in any way which means that sys.stdout or the print statement or function will still not provide unicode support.

  Changed in version 4.0: Added the color flag.

  New in version 3.0: The err parameter was added.

  Changed in version 2.0: Starting with version 2.0 of Click, the echo function will work with colorama if it’s installed.

  Parameters

    - message – the message to print

    - file – the file to write to (defaults to `stdout`)

    - err – if set to true the file defaults to `stderr` instead of `stdout`. This is faster and easier than calling `get_text_stderr()` yourself.

    - nl – if set to True (the default) a newline is printed afterwards.

    - color – controls if the terminal supports ANSI colors or not. The default is autodetection.

`asyncclick.` `echo_via_pager`(text_or_generator, color=None)

  This function takes a text and shows it via an environment specific pager on stdout.

  Changelog

  Changed in version 3.0: Added the color flag.

  Parameters

    - text_or_generator – the text to page, or alternatively, a generator emitting the text to page.

    - color – controls if the pager supports ANSI colors or not. The default is autodetection.

`asyncclick.` `prompt`(text, default=None, hide_input=False, confirmation_prompt=False, type=None, value_proc=None, prompt_suffix=': ', show_default=True, err=False, show_choices=True)

  Prompts a user for input. This is a convenience function that can be used to prompt a user for input later.

  If the user aborts the input by sending a interrupt signal, this function will catch it and raise a `Abort` exception.

  Changelog

  New in version 7.0: Added the show_choices parameter.

  New in version 6.0: Added unicode support for cmd.exe on Windows.

  New in version 4.0: Added the err parameter.

  Parameters

    - text – the text to show for the prompt.

    - default – the default value to use if no input happens. If this is not given it will prompt until it’s aborted.

    - hide_input – if this is set to true then the input value will be hidden.

    - confirmation_prompt – asks for confirmation for the value.

    - type – the type to use to check the value against.

    - value_proc – if this parameter is provided it’s a function that is invoked instead of the type conversion to convert a value.

    - prompt_suffix – a suffix that should be added to the prompt.

    - show_default – shows or hides the default value in the prompt.

    - err – if set to true the file defaults to `stderr` instead of `stdout`, the same as with echo.

    - show_choices – Show or hide choices if the passed type is a Choice. For example if type is a Choice of either day or week, show_choices is true and text is “Group by” then the prompt will be “Group by (day, week): “.

`asyncclick.` `confirm`(text, default=False, abort=False, prompt_suffix=': ', show_default=True, err=False)

  Prompts for confirmation (yes/no question).

  If the user aborts the input by sending a interrupt signal this function will catch it and raise a `Abort` exception.

  Changelog

  New in version 4.0: Added the err parameter.

  Parameters

    - text – the question to ask.

    - default – the default for the prompt.

    - abort – if this is set to True a negative answer aborts the exception by raising `Abort`.

    - prompt_suffix – a suffix that should be added to the prompt.

    - show_default – shows or hides the default value in the prompt.

    - err – if set to true the file defaults to `stderr` instead of `stdout`, the same as with echo.

`asyncclick.` `progressbar`(iterable=None, length=None, label=None, show_eta=True, show_percent=None, show_pos=False, item_show_func=None, fill_char='#', empty_char='-', bar_template='%(label)s [%(bar)s] %(info)s', info_sep=' ', width=36, file=None, color=None, update_min_steps=1)

  This function creates an iterable context manager that can be used to iterate over something while showing a progress bar. It will either iterate over the iterable or length items (that are counted up). While iteration happens, this function will print a rendered progress bar to the given file (defaults to stdout) and will attempt to calculate remaining time and more. By default, this progress bar will not be rendered if the file is not a terminal.

  The context manager creates the progress bar. When the context manager is entered the progress bar is already created. With every iteration over the progress bar, the iterable passed to the bar is advanced and the bar is updated. When the context manager exits, a newline is printed and the progress bar is finalized on screen.

  Note: The progress bar is currently designed for use cases where the total progress can be expected to take at least several seconds. Because of this, the ProgressBar class object won’t display progress that is considered too fast, and progress where the time between steps is less than a second.

  No printing must happen or the progress bar will be unintentionally destroyed.

  Example usage:

      with progressbar(items) as bar:
          for item in bar:
              do_something_with(item)

  Alternatively, if no iterable is specified, one can manually update the progress bar through the update() method instead of directly iterating over the progress bar. The update method accepts the number of steps to increment the bar with:

      with progressbar(length=chunks.total_bytes) as bar:
          for chunk in chunks:
              process_chunk(chunk)
              bar.update(chunks.bytes)

  The `update()` method also takes an optional value specifying the `current_item` at the new position. This is useful when used together with `item_show_func` to customize the output for each manual step:

      with click.progressbar(
          length=total_size,
          label='Unzipping archive',
          item_show_func=lambda a: a.filename
      ) as bar:
          for archive in zip_file:
              archive.extract()
              bar.update(archive.size, archive)

  Parameters

    - iterable – an iterable to iterate over. If not provided the length is required.

    - length – the number of items to iterate over. By default the progressbar will attempt to ask the iterator about its length, which might or might not work. If an iterable is also provided this parameter can be used to override the length. If an iterable is not provided the progress bar will iterate over a range of that length.

    - label – the label to show next to the progress bar.

    - show_eta – enables or disables the estimated time display. This is automatically disabled if the length cannot be determined.

    - show_percent – enables or disables the percentage display. The default is True if the iterable has a length or False if not.

    - show_pos – enables or disables the absolute position display. The default is False.

    - item_show_func – a function called with the current item which can return a string to show the current item next to the progress bar. Note that the current item can be None!

    - fill_char – the character to use to show the filled part of the progress bar.

    - empty_char – the character to use to show the non-filled part of the progress bar.

    - bar_template – the format string to use as template for the bar. The parameters in it are `label` for the label, `bar` for the progress bar and `info` for the info section.

    - info_sep – the separator between multiple info items (eta etc.)

    - width – the width of the progress bar in characters, 0 means full terminal width

    - file – the file to write to. If this is not a terminal then only the label is printed.

    - color – controls if the terminal supports ANSI colors or not. The default is autodetection. This is only needed if ANSI codes are included anywhere in the progress bar output which is not the case by default.

    - update_min_steps – Render only when this many updates have completed. This allows tuning for very fast iterators.

  New in version 8.0: Added the `update_min_steps` parameter.

  Changelog

  Changed in version 4.0: Added the `color` parameter. Added the `update` method to the object.

  New in version 2.0.

`asyncclick.` `clear`()

  Clears the terminal screen. This will have the effect of clearing the whole visible space of the terminal and moving the cursor to the top left. This does not do anything if not connected to a terminal.

  Changelog

  New in version 2.0.

`asyncclick.` `style`(text, fg=None, bg=None, bold=None, dim=None, underline=None, blink=None, reverse=None, reset=True)

  Styles a text with ANSI styles and returns the new string. By default the styling is self contained which means that at the end of the string a reset code is issued. This can be prevented by passing `reset=False`.

  Examples:

      click.echo(click.style('Hello World!', fg='green'))
      click.echo(click.style('ATTENTION!', blink=True))
      click.echo(click.style('Some things', reverse=True, fg='cyan'))
      click.echo(click.style('More colors', fg=(255, 12, 128), bg=117))

  Supported color names:

  - `black` (might be a gray)

  - `red`

  - `green`

  - `yellow` (might be an orange)

  - `blue`

  - `magenta`

  - `cyan`

  - `white` (might be light gray)

  - `bright_black`

  - `bright_red`

  - `bright_green`

  - `bright_yellow`

  - `bright_blue`

  - `bright_magenta`

  - `bright_cyan`

  - `bright_white`

  - `reset` (reset the color code only)

  If the terminal supports it, color may also be specified as:

  - An integer in the interval [0, 255]. The terminal must support 8-bit/256-color mode.

  - An RGB tuple of three integers in [0, 255]. The terminal must support 24-bit/true-color mode.

  See https://en.wikipedia.org/wiki/ANSI_color and https://gist.github.com/XVilka/8346728 for more information.

  Parameters

    - text – the string to style with ansi codes.

    - fg – if provided this will become the foreground color.

    - bg – if provided this will become the background color.

    - bold – if provided this will enable or disable bold mode.

    - dim – if provided this will enable or disable dim mode. This is badly supported.

    - underline – if provided this will enable or disable underline.

    - blink – if provided this will enable or disable blinking.

    - reverse – if provided this will enable or disable inverse rendering (foreground becomes background and the other way round).

    - reset – by default a reset-all code is added at the end of the string which means that styles do not carry over. This can be disabled to compose styles.

  Changed in version 8.0: A non-string `message` is converted to a string.

  Changed in version 8.0: Added support for 256 and RGB color codes.

  Changelog

  Changed in version 7.0: Added support for bright colors.

  New in version 2.0.

`asyncclick.` `unstyle`(text)

  Removes ANSI styling information from a string. Usually it’s not necessary to use this function as Click’s echo function will automatically remove styling if necessary.

  Changelog

  New in version 2.0.

  Parameters

    text – the text to remove style information from.

`asyncclick.` `secho`(message=None, file=None, nl=True, err=False, color=None, **styles)

  This function combines `echo()` and `style()` into one call. As such the following two calls are the same:

      click.secho('Hello World!', fg='green')
      click.echo(click.style('Hello World!', fg='green'))

  All keyword arguments are forwarded to the underlying functions depending on which one they go with.

  Non-string types will be converted to `str`. However, `bytes` are passed directly to `echo()` without applying style. If you want to style bytes that represent text, call `bytes.decode()` first.

  Changed in version 8.0: A non-string `message` is converted to a string. Bytes are passed through without style applied.

  Changelog

  New in version 2.0.

`asyncclick.` `edit`(text=None, editor=None, env=None, require_save=True, extension='.txt', filename=None)

  Edits the given text in the defined editor. If an editor is given (should be the full path to the executable but the regular operating system search path is used for finding the executable) it overrides the detected editor. Optionally, some environment variables can be used. If the editor is closed without changes, None is returned. In case a file is edited directly the return value is always None and require_save and extension are ignored.

  If the editor cannot be opened a `UsageError` is raised.

  Note for Windows: to simplify cross-platform usage, the newlines are automatically converted from POSIX to Windows and vice versa. As such, the message here will have `\n` as newline markers.

  Parameters

    - text – the text to edit.

    - editor – optionally the editor to use. Defaults to automatic detection.

    - env – environment variables to forward to the editor.

    - require_save – if this is true, then not saving in the editor will make the return value become None.

    - extension – the extension to tell the editor about. This defaults to .txt but changing this might change syntax highlighting.

    - filename – if provided it will edit this file instead of the provided text contents. It will not use a temporary file as an indirection in that case.

`asyncclick.` `launch`(url, wait=False, locate=False)

  This function launches the given URL (or filename) in the default viewer application for this file type. If this is an executable, it might launch the executable in a new session. The return value is the exit code of the launched application. Usually, `0` indicates success.

  Examples:

      click.launch('https://click.palletsprojects.com/')
      click.launch('/my/downloaded/file', locate=True)

  Changelog

  New in version 2.0.

  Parameters

    - url – URL or filename of the thing to launch.

    - wait – Wait for the program to exit before returning. This only works if the launched program blocks. In particular, `xdg-open` on Linux does not block.

    - locate – if this is set to True then instead of launching the application associated with the URL it will attempt to launch a file manager with the file located. This might have weird effects if the URL does not point to the filesystem.

`asyncclick.` `getchar`(echo=False)

  Fetches a single character from the terminal and returns it. This will always return a unicode character and under certain rare circumstances this might return more than one character. The situations which more than one character is returned is when for whatever reason multiple characters end up in the terminal buffer or standard input was not actually a terminal.

  Note that this will always read from the terminal, even if something is piped into the standard input.

  Note for Windows: in rare cases when typing non-ASCII characters, this function might wait for a second character and then return both at once. This is because certain Unicode characters look like special-key markers.

  Changelog

  New in version 2.0.

  Parameters

    echo – if set to True, the character read will also show up on the terminal. The default is to not show it.

`asyncclick.` `pause`(info='Press any key to continue ...', err=False)

  This command stops execution and waits for the user to press any key to continue. This is similar to the Windows batch “pause” command. If the program is not run through a terminal, this command will instead do nothing.

  Changelog

  New in version 4.0: Added the err parameter.

  New in version 2.0.

  Parameters

    - info – the info string to print before pausing.

    - err – if set to message goes to `stderr` instead of `stdout`, the same as with echo.

`asyncclick.` `get_terminal_size`()

  Returns the current size of the terminal as tuple in the form `(width, height)` in columns and rows.

`asyncclick.` `get_binary_stream`(name)

  Returns a system stream for byte processing.

  Parameters

    name – the name of the stream to open. Valid names are `'stdin'`, `'stdout'` and `'stderr'`

`asyncclick.` `get_text_stream`(name, encoding=None, errors='strict')

  Returns a system stream for text processing. This usually returns a wrapped stream around a binary stream returned from `get_binary_stream()` but it also can take shortcuts for already correctly configured streams.

  Parameters

    - name – the name of the stream to open. Valid names are `'stdin'`, `'stdout'` and `'stderr'`

    - encoding – overrides the detected default encoding.

    - errors – overrides the default error mode.

`asyncclick.` `open_file`(filename, mode='r', encoding=None, errors='strict', lazy=False, atomic=False)

  This is similar to how the `File` works but for manual usage. Files are opened non lazy by default. This can open regular files as well as stdin/stdout if `'-'` is passed.

  If stdin/stdout is returned the stream is wrapped so that the context manager will not close the stream accidentally. This makes it possible to always use the function like this without having to worry to accidentally close a standard stream:

      with open_file(filename) as f:
          ...

  Changelog

  New in version 3.0.

  Parameters

    - filename – the name of the file to open (or `'-'` for stdin/stdout).

    - mode – the mode in which to open the file.

    - encoding – the encoding to use.

    - errors – the error handling for this file.

    - lazy – can be flipped to true to open the file lazily.

    - atomic – in atomic mode writes go into a temporary file and it’s moved on close.

`asyncclick.` `get_app_dir`(app_name, roaming=True, force_posix=False)

  Returns the config folder for the application. The default behavior is to return whatever is most appropriate for the operating system.

  To give you an idea, for an app called `"Foo Bar"`, something like the following folders could be returned:

  Mac OS X:

    `~/Library/Application Support/Foo Bar`

  Mac OS X (POSIX):

    `~/.foo-bar`

  Unix:

    `~/.config/foo-bar`

  Unix (POSIX):

    `~/.foo-bar`

  Win XP (roaming):

    `C:\Documents and Settings\<user>\Local Settings\Application Data\Foo Bar`

  Win XP (not roaming):

    `C:\Documents and Settings\<user>\Application Data\Foo Bar`

  Win 7 (roaming):

    `C:\Users\<user>\AppData\Roaming\Foo Bar`

  Win 7 (not roaming):

    `C:\Users\<user>\AppData\Local\Foo Bar`

  Changelog

  New in version 2.0.

  Parameters

    - app_name – the application name. This should be properly capitalized and can contain whitespace.

    - roaming – controls if the folder should be roaming or not on Windows. Has no affect otherwise.

    - force_posix – if this is set to True then on any POSIX system the folder will be stored in the home folder with a leading dot instead of the XDG config home or darwin’s application support folder.

`asyncclick.` `format_filename`(filename, shorten=False)

  Formats a filename for user display. The main purpose of this function is to ensure that the filename can be displayed at all. This will decode the filename to unicode if necessary in a way that it will not fail. Optionally, it can shorten the filename to not include the full path to the filename.

  Parameters

    - filename – formats a filename for UI display. This will also convert the filename into unicode without failing.

    - shorten – this optionally shortens the filename to strip of the path that leads up to it.

## Commands

class `asyncclick.` `BaseCommand`(name, context_settings=None)

  The base command implements the minimal API contract of commands. Most code will never use this as it does not implement a lot of useful functionality but it can act as the direct subclass of alternative parsing methods that do not depend on the Click parser.

  For instance, this can be used to bridge Click and other systems like argparse or docopt.

  Because base commands do not implement a lot of the API that other parts of Click take for granted, they are not supported for all operations. For instance, they cannot be used with the decorators usually and they have no built-in callback system.

  Changelog

  Changed in version 2.0: Added the context_settings parameter.

  Parameters

    - name – the name of the command to use unless a group overrides it.

    - context_settings – an optional dictionary with defaults that are passed to the context object.

  `allow_extra_args` = False

    the default for the `Context.allow_extra_args` flag.

  `allow_interspersed_args` = True

    the default for the `Context.allow_interspersed_args` flag.

  `context_class`

    The context class to create with `make_context()`.

    New in version 8.0.

    alias of `asyncclick.core.Context`

  `context_settings`

    an optional dictionary with defaults passed to the context.

  `ignore_unknown_options` = False

    the default for the `Context.ignore_unknown_options` flag.

  async `invoke`(ctx)

    Given a context, this invokes the command. The default implementation is raising a not implemented error.

  async `main`(args=None, prog_name=None, complete_var=None, standalone_mode=True, **extra)

    This is the way to invoke a script with all the bells and whistles as a command line application. This will always terminate the application after a call. If this is not wanted, `SystemExit` needs to be caught.

    This method is also available by directly calling the instance of a `Command`.

    Changelog

    New in version 3.0: Added the standalone_mode flag to control the standalone mode.

    Parameters

      - args – the arguments that should be used for parsing. If not provided, `sys.argv[1:]` is used.

      - prog_name – the program name that should be used. By default the program name is constructed by taking the file name from `sys.argv[0]`.

      - complete_var – the environment variable that controls the bash completion support. The default is `"_<prog_name>_COMPLETE"` with prog_name in uppercase.

      - standalone_mode – the default behavior is to invoke the script in standalone mode. Click will then handle exceptions and convert them into error messages and the function will never return but shut down the interpreter. If this is set to False they will be propagated to the caller and the return value of this function is the return value of `invoke()`.

      - extra – extra keyword arguments are forwarded to the context constructor. See `Context` for more information.

  async `make_context`(info_name, args, parent=None, **extra)

    This function when given an info name and arguments will kick off the parsing and create a new `Context`. It does not invoke the actual command callback though.

    To quickly customize the context class used without overriding this method, set the `context_class` attribute.

    Parameters

      - info_name – the info name for this invokation. Generally this is the most descriptive name for the script or command. For the toplevel script it’s usually the name of the script, for commands below it it’s the name of the script.

      - args – the arguments to parse as list of strings.

      - parent – the parent context if available.

      - extra – extra keyword arguments forwarded to the context constructor.

    Changed in version 8.0: Added the `context_class` attribute.

  `name`

    the name the command thinks it has. Upon registering a command on a `Group` the group will default the command name with this information. You should instead use the `Context`'s `info_name` attribute.

  async `parse_args`(ctx, args)

    Given a context and a list of arguments this creates the parser and parses the arguments, then modifies the context as necessary. This is automatically invoked by `make_context()`.

  `shell_complete`(ctx, incomplete)

    Return a list of completions for the incomplete value. Looks at the names of chained multi-commands.

    Any command could be part of a chained multi-command, so sibling commands are valid at any point during command completion. Other command classes will return more completions.

    Parameters

      - ctx – Invocation context for this command.

      - incomplete – Value being completed. May be empty.

    New in version 8.0.

  async `to_info_dict`(ctx)

    Gather information that could be useful for a tool generating user-facing documentation. This traverses the entire structure below this command.

    Use `click.Context.to_info_dict()` to traverse the entire CLI structure.

    Parameters

      ctx – A `Context` representing this command.

    New in version 8.0.

class `asyncclick.` `Command`(name, context_settings=None, callback=None, params=None, help=None, epilog=None, short_help=None, options_metavar='[OPTIONS]', add_help_option=True, no_args_is_help=False, hidden=False, deprecated=False)

  Commands are the basic building block of command line interfaces in Click. A basic command handles command line parsing and might dispatch more parsing to commands nested below it.

  Changed in version 8.0: Added repr showing the command name

  Changelog

  Changed in version 7.1: Added the no_args_is_help parameter.

  Changed in version 2.0: Added the context_settings parameter.

  Parameters

    - name – the name of the command to use unless a group overrides it.

    - context_settings – an optional dictionary with defaults that are passed to the context object.

    - callback – the callback to invoke. This is optional.

    - params – the parameters to register with this command. This can be either `Option` or `Argument` objects.

    - help – the help string to use for this command.

    - epilog – like the help string but it’s printed at the end of the help page after everything else.

    - short_help – the short help to use for this command. This is shown on the command listing of the parent command.

    - add_help_option – by default each command registers a `--help` option. This can be disabled by this parameter.

    - no_args_is_help – this controls what happens if no arguments are provided. This option is disabled by default. If enabled this will add `--help` as argument if no arguments are passed

    - hidden – hide this command from help outputs.

    - deprecated – issues a message indicating that the command is deprecated.

  `callback`

    the callback to execute when the command fires. This might be None in which case nothing happens.

  `collect_usage_pieces`(ctx)

    Returns all the pieces that go into the usage line and returns it as a list of strings.

  `format_epilog`(ctx, formatter)

    Writes the epilog into the formatter if it exists.

  `format_help`(ctx, formatter)

    Writes the help into the formatter if it exists.

    This is a low-level method called by `get_help()`.

    This calls the following methods:

    - `format_usage()`

    - `format_help_text()`

    - `format_options()`

    - `format_epilog()`

  `format_help_text`(ctx, formatter)

    Writes the help text to the formatter if it exists.

  `format_options`(ctx, formatter)

    Writes all the options into the formatter if they exist.

  `format_usage`(ctx, formatter)

    Writes the usage line into the formatter.

    This is a low-level method called by `get_usage()`.

  `get_help`(ctx)

    Formats the help into a string and returns it.

    Calls `format_help()` internally.

  `get_help_option`(ctx)

    Returns the help option object.

  `get_help_option_names`(ctx)

    Returns the names for the help option.

  `get_short_help_str`(limit=45)

    Gets short help for the command or makes it by shortening the long help string.

  `get_usage`(ctx)

    Formats the usage line into a string and returns it.

    Calls `format_usage()` internally.

  async `invoke`(ctx)

    Given a context, this invokes the attached callback (if it exists) in the right way.

  `make_parser`(ctx)

    Creates the underlying option parser for this command.

  `params`

    the list of parameters for this command in the order they should show up in the help page and execute. Eager parameters will automatically be handled before non eager ones.

  async `parse_args`(ctx, args)

    Given a context and a list of arguments this creates the parser and parses the arguments, then modifies the context as necessary. This is automatically invoked by `make_context()`.

  `shell_complete`(ctx, incomplete)

    Return a list of completions for the incomplete value. Looks at the names of options and chained multi-commands.

    Parameters

      - ctx – Invocation context for this command.

      - incomplete – Value being completed. May be empty.

    New in version 8.0.

  async `to_info_dict`(ctx)

    Gather information that could be useful for a tool generating user-facing documentation. This traverses the entire structure below this command.

    Use `click.Context.to_info_dict()` to traverse the entire CLI structure.

    Parameters

      ctx – A `Context` representing this command.

    New in version 8.0.

class `asyncclick.` `MultiCommand`(name=None, invoke_without_command=False, no_args_is_help=None, subcommand_metavar=None, chain=False, result_callback=None, **attrs)

  A multi command is the basic implementation of a command that dispatches to subcommands. The most common version is the `Group`.

  Parameters

    - invoke_without_command – this controls how the multi command itself is invoked. By default it’s only invoked if a subcommand is provided.

    - no_args_is_help – this controls what happens if no arguments are provided. This option is enabled by default if invoke_without_command is disabled or disabled if it’s enabled. If enabled this will add `--help` as argument if no arguments are passed.

    - subcommand_metavar – the string that is used in the documentation to indicate the subcommand place.

    - chain – if this is set to True chaining of multiple subcommands is enabled. This restricts the form of commands in that they cannot have optional arguments but it allows multiple commands to be chained together.

    - result_callback – the result callback to attach to this multi command.

  `collect_usage_pieces`(ctx)

    Returns all the pieces that go into the usage line and returns it as a list of strings.

  `format_commands`(ctx, formatter)

    Extra format methods for multi methods that adds all the commands after the options.

  `format_options`(ctx, formatter)

    Writes all the options into the formatter if they exist.

  `get_command`(ctx, cmd_name)

    Given a context and a command name, this returns a `Command` object if it exists or returns None.

  async `invoke`(ctx)

    Given a context, this invokes the attached callback (if it exists) in the right way.

  `list_commands`(ctx)

    Returns a list of subcommand names in the order they should appear.

  async `parse_args`(ctx, args)

    Given a context and a list of arguments this creates the parser and parses the arguments, then modifies the context as necessary. This is automatically invoked by `make_context()`.

  `result_callback`

    The result callback that is stored. This can be set or overridden with the `resultcallback()` decorator.

  `resultcallback`(replace=False)

    Adds a result callback to the command. By default if a result callback is already registered this will chain them but this can be disabled with the replace parameter. The result callback is invoked with the return value of the subcommand (or the list of return values from all subcommands if chaining is enabled) as well as the parameters as they would be passed to the main callback.

    Example:

        @click.group()
        @click.option('-i', '--input', default=23)
        def cli(input):
            return 42

        @cli.resultcallback()
        def process_result(result, input):
            return result + input

    Parameters

      replace – if set to True an already existing result callback will be removed.

    Changelog

    New in version 3.0.

  `shell_complete`(ctx, incomplete)

    Return a list of completions for the incomplete value. Looks at the names of options, subcommands, and chained multi-commands.

    Parameters

      - ctx – Invocation context for this command.

      - incomplete – Value being completed. May be empty.

    New in version 8.0.

  async `to_info_dict`(ctx)

    Gather information that could be useful for a tool generating user-facing documentation. This traverses the entire structure below this command.

    Use `click.Context.to_info_dict()` to traverse the entire CLI structure.

    Parameters

      ctx – A `Context` representing this command.

    New in version 8.0.

class `asyncclick.` `Group`(name=None, commands=None, **attrs)

  A group allows a command to have subcommands attached. This is the most common way to implement nesting in Click.

  Parameters

    - name – The name of the group command.

    - commands – A dict mapping names to `Command` objects. Can also be a list of `Command`, which will use `Command.name` to create the dict.

    - attrs – Other command arguments described in `MultiCommand`, `Command`, and `BaseCommand`.

  Changed in version 8.0: The `commmands` argument can be a list of command objects.

  `add_command`(cmd, name=None)

    Registers another `Command` with this group. If the name is not provided, the name of the command is used.

  `command`(*args, **kwargs)

    A shortcut decorator for declaring and attaching a command to the group. This takes the same arguments as `command()` and immediately registers the created command with this group by calling `add_command()`.

    To customize the command class used, set the `command_class` attribute.

    Changed in version 8.0: Added the `command_class` attribute.

  `command_class` = None

    If set, this is used by the group’s `command()` decorator as the default `Command` class. This is useful to make all subcommands use a custom command class.

    New in version 8.0.

  `commands`

    The registered subcommands by their exported names.

  `get_command`(ctx, cmd_name)

    Given a context and a command name, this returns a `Command` object if it exists or returns None.

  `group`(*args, **kwargs)

    A shortcut decorator for declaring and attaching a group to the group. This takes the same arguments as `group()` and immediately registers the created group with this group by calling `add_command()`.

    To customize the group class used, set the `group_class` attribute.

    Changed in version 8.0: Added the `group_class` attribute.

  `group_class` = None

    If set, this is used by the group’s `group()` decorator as the default `Group` class. This is useful to make all subgroups use a custom group class.

    If set to the special value `type` (literally `group_class = type`), this group’s class will be used as the default class. This makes a custom group class continue to make custom groups.

    New in version 8.0.

  `list_commands`(ctx)

    Returns a list of subcommand names in the order they should appear.

class `asyncclick.` `CommandCollection`(name=None, sources=None, **attrs)

  A command collection is a multi command that merges multiple multi commands together into one. This is a straightforward implementation that accepts a list of different multi commands as sources and provides all the commands for each of them.

  `add_source`(multi_cmd)

    Adds a new multi command to the chain dispatcher.

  `get_command`(ctx, cmd_name)

    Given a context and a command name, this returns a `Command` object if it exists or returns None.

  `list_commands`(ctx)

    Returns a list of subcommand names in the order they should appear.

  `sources`

    The list of registered multi commands.

## Parameters

class `asyncclick.` `Parameter`(param_decls=None, type=None, required=False, default=None, callback=None, nargs=None, metavar=None, expose_value=True, is_eager=False, envvar=None, shell_complete=None, autocompletion=None)

  A parameter to a command comes in two versions: they are either `Option`s or `Argument`s. Other subclasses are currently not supported by design as some of the internals for parsing are intentionally not finalized.

  Some settings are supported by both options and arguments.

  Parameters

    - param_decls – the parameter declarations for this option or argument. This is a list of flags or argument names.

    - type – the type that should be used. Either a `ParamType` or a Python type. The later is converted into the former automatically if supported.

    - required – controls if this is optional or not.

    - default – the default value if omitted. This can also be a callable, in which case it’s invoked when the default is needed without any arguments.

    - callback – a callback that should be executed after the parameter was matched. This is called as `fn(ctx, param, value)` and needs to return the value.

    - nargs – the number of arguments to match. If not `1` the return value is a tuple instead of single value. The default for nargs is `1` (except if the type is a tuple, then it’s the arity of the tuple). If `nargs=-1`, all remaining parameters are collected.

    - metavar – how the value is represented in the help page.

    - expose_value – if this is True then the value is passed onwards to the command callback and stored on the context, otherwise it’s skipped.

    - is_eager – eager values are processed before non eager ones. This should not be set for arguments or it will inverse the order of processing.

    - envvar – a string or list of strings that are environment variables that should be checked.

    - shell_complete – A function that returns custom shell completions. Used instead of the param’s type completion if given. Takes `ctx, param, incomplete` and must return a list of `CompletionItem` or a list of strings.

  Changed in version 8.0: `autocompletion` is renamed to `shell_complete` and has new semantics described above. The old name is deprecated and will be removed in 8.1, until then it will be wrapped to match the new requirements.

  Changed in version 8.0: For `multiple=True, nargs>1`, the default must be a list of tuples.

  Changed in version 8.0: Setting a default is no longer required for `nargs>1`, it will default to `None`. `multiple=True` or `nargs=-1` will default to `()`.

  Changelog

  Changed in version 7.1: Empty environment variables are ignored rather than taking the empty string value. This makes it possible for scripts to clear variables if they can’t unset them.

  Changed in version 2.0: Changed signature for parameter callback to also be passed the parameter. The old callback format will still work, but it will raise a warning to give you a chance to migrate the code easier.

  `get_default`(ctx, call=True)

    Get the default for the parameter. Tries `Context.lookup_value()` first, then the local default.

    Parameters

      - ctx – Current context.

      - call – If the default is a callable, call it. Disable to return the callable instead.

    Changed in version 8.0: Looks at `ctx.default_map` first.

    Changed in version 8.0: Added the `call` parameter.

  `get_error_hint`(ctx)

    Get a stringified version of the param for use in error messages to indicate which param caused the error.

  property `human_readable_name`

    Returns the human readable name of this parameter. This is the same as the name for options, but the metavar for arguments.

  `process_value`(ctx, value)

    Given a value and context this runs the logic to convert the value as necessary.

  `shell_complete`(ctx, incomplete)

    Return a list of completions for the incomplete value. If a `shell_complete` function was given during init, it is used. Otherwise, the `type` `shell_complete()` function is used.

    Parameters

      - ctx – Invocation context for this command.

      - incomplete – Value being completed. May be empty.

    New in version 8.0.

  async `to_info_dict`()

    Gather information that could be useful for a tool generating user-facing documentation.

    Use `click.Context.to_info_dict()` to traverse the entire CLI structure.

    New in version 8.0.

  `type_cast_value`(ctx, value)

    Given a value this runs it properly through the type system. This automatically handles things like nargs and multiple as well as composite types.

class `asyncclick.` `Option`(param_decls=None, show_default=False, prompt=False, confirmation_prompt=False, prompt_required=True, hide_input=False, is_flag=None, flag_value=None, multiple=False, count=False, allow_from_autoenv=True, type=None, help=None, hidden=False, show_choices=True, show_envvar=False, **attrs)

  Options are usually optional values on the command line and have some extra features that arguments don’t have.

  All other parameters are passed onwards to the parameter constructor.

  Parameters

    - show_default – controls if the default value should be shown on the help page. Normally, defaults are not shown. If this value is a string, it shows the string instead of the value. This is particularly useful for dynamic options.

    - show_envvar – controls if an environment variable should be shown on the help page. Normally, environment variables are not shown.

    - prompt – if set to True or a non empty string then the user will be prompted for input. If set to True the prompt will be the option name capitalized.

    - confirmation_prompt – if set then the value will need to be confirmed if it was prompted for.

    - prompt_required – If set to `False`, the user will be prompted for input only when the option was specified as a flag without a value.

    - hide_input – if this is True then the input on the prompt will be hidden from the user. This is useful for password input.

    - is_flag – forces this option to act as a flag. The default is auto detection.

    - flag_value – which value should be used for this flag if it’s enabled. This is set to a boolean automatically if the option string contains a slash to mark two options.

    - multiple – if this is set to True then the argument is accepted multiple times and recorded. This is similar to `nargs` in how it works but supports arbitrary number of arguments.

    - count – this flag makes an option increment an integer.

    - allow_from_autoenv – if this is enabled then the value of this parameter will be pulled from an environment variable in case a prefix is defined on the context.

    - help – the help string.

    - hidden – hide this option from help outputs.

class `asyncclick.` `Argument`(param_decls, required=None, **attrs)

  Arguments are positional parameters to a command. They generally provide fewer features than options but can have infinite `nargs` and are required by default.

  All parameters are passed onwards to the parameter constructor.

## Context

class `asyncclick.` `Context`(command, parent=None, info_name=None, obj=None, auto_envvar_prefix=None, default_map=None, terminal_width=None, max_content_width=None, resilient_parsing=False, allow_extra_args=None, allow_interspersed_args=None, ignore_unknown_options=None, help_option_names=None, token_normalize_func=None, color=None, show_default=None)

  The context is a special internal object that holds state relevant for the script execution at every single level. It’s normally invisible to commands unless they opt-in to getting access to it.

  The context is useful as it can pass internal objects around and can control special execution features such as reading data from environment variables.

  A context can be used as async context manager in which case it will close any managed contexts, and call `close()` on teardown.

  Parameters

    - command – the command class for this context.

    - parent – the parent context.

    - info_name – the info name for this invocation. Generally this is the most descriptive name for the script or command. For the toplevel script it is usually the name of the script, for commands below it it’s the name of the script.

    - obj – an arbitrary object of user data.

    - auto_envvar_prefix – the prefix to use for automatic environment variables. If this is None then reading from environment variables is disabled. This does not affect manually set environment variables which are always read.

    - default_map – a dictionary (like object) with default values for parameters.

    - terminal_width – the width of the terminal. The default is inherit from parent context. If no context defines the terminal width then auto detection will be applied.

    - max_content_width – the maximum width for content rendered by Click (this currently only affects help pages). This defaults to 80 characters if not overridden. In other words: even if the terminal is larger than that, Click will not format things wider than 80 characters by default. In addition to that, formatters might add some safety mapping on the right.

    - resilient_parsing – if this flag is enabled then Click will parse without any interactivity or callback invocation. Default values will also be ignored. This is useful for implementing things such as completion support.

    - allow_extra_args – if this is set to True then extra arguments at the end will not raise an error and will be kept on the context. The default is to inherit from the command.

    - allow_interspersed_args – if this is set to False then options and arguments cannot be mixed. The default is to inherit from the command.

    - ignore_unknown_options – instructs click to ignore options it does not know and keeps them for later processing.

    - help_option_names – optionally a list of strings that define how the default help parameter is named. The default is `['--help']`.

    - token_normalize_func – an optional function that is used to normalize tokens (options, choices, etc.). This for instance can be used to implement case insensitive behavior.

    - color – controls if the terminal supports ANSI colors or not. The default is autodetection. This is only needed if ANSI codes are used in texts that Click prints which is by default not the case. This for instance would affect help output.

    - show_default – Show defaults for all options. If not set, defaults to the value from a parent context. Overrides an option’s `show_default` argument.

  Changed in version 8.0: The `show_default` parameter defaults to the value from the parent context.

  Changelog

  Changed in version 7.1: Added the `show_default` parameter.

  Changed in version 4.0: Added the `color`, `ignore_unknown_options`, and `max_content_width` parameters.

  Changed in version 3.0: Added the `allow_extra_args` and `allow_interspersed_args` parameters.

  Changed in version 2.0: Added the `resilient_parsing`, `help_option_names`, and `token_normalize_func` parameters.

  `abort`()

    Aborts the script.

  async `aclose`()

    Invoke all close callbacks registered with `call_on_close()`, and exit all context managers entered with `with_resource()`.

  `allow_extra_args`

    Indicates if the context allows extra args or if it should fail on parsing.

    Changelog

    New in version 3.0.

  `allow_interspersed_args`

    Indicates if the context allows mixing of arguments and options or not.

    Changelog

    New in version 3.0.

  `args`

    the leftover arguments.

  `call_on_close`(f)

    Register a function to be called when the context tears down.

    This can be used to close resources opened during the script execution. Resources that support Python’s context manager protocol which would be used in a `with` statement should be registered with `with_resource()` instead.

    Parameters

      f – The function to execute on teardown.

  `color`

    Controls if styling output is wanted or not.

  `command`

    the `Command` for this context.

  property `command_path`

    The computed command path. This is used for the `usage` information on the help page. It’s automatically created by combining the info names of the chain of contexts to the root.

  `ensure_object`(object_type)

    Like `find_object()` but sets the innermost object to a new instance of object_type if it does not exist.

  `exit`(code=0)

    Exits the application with a given exit code.

  `fail`(message)

    Aborts the execution of the program with a specific error message.

    Parameters

      message – the error message to fail with.

  `find_object`(object_type)

    Finds the closest object of a given type.

  `find_root`()

    Finds the outermost context.

  `formatter_class`

    alias of `asyncclick.formatting.HelpFormatter`

  async `forward`(**kwargs)

    Similar to `invoke()` but fills in default keyword arguments from the current context if the other command expects it. This cannot invoke callbacks directly, only other commands.

  `get_help`()

    Helper method to get formatted help page for the current context and command.

  `get_parameter_source`(name)

    Get the source of a parameter. This indicates the location from which the value of the parameter was obtained.

    This can be useful for determining when a user specified a value on the command line that is the same as the default value. It will be `DEFAULT` only if the value was actually taken from the default.

    Parameters

      name – The name of the parameter.

    Return type

      ParameterSource

    Changed in version 8.0: Returns `None` if the parameter was not provided from any source.

  `get_usage`()

    Helper method to get formatted usage string for the current context and command.

  `help_option_names`

    The names for the help options.

  `ignore_unknown_options`

    Instructs click to ignore options that a command does not understand and will store it on the context for later processing. This is primarily useful for situations where you want to call into external programs. Generally this pattern is strongly discouraged because it’s not possibly to losslessly forward all arguments.

    Changelog

    New in version 4.0.

  `info_name`

    the descriptive information name

  async `invoke`(**kwargs)

    Invokes a command callback in exactly the way it expects. There are two ways to invoke this method:

    1. the first argument can be a callback and all other arguments and keyword arguments are forwarded directly to the function.

    2. the first argument is a click command object. In that case all arguments are forwarded as well but proper click parameters (options and click arguments) must be keyword arguments and Click will fill in defaults.

    Note that before Click 3.2 keyword arguments were not properly filled in against the intention of this code and no context was created. For more information about this change and why it was done in a bugfix release see Upgrading to 3.2.

  `invoked_subcommand`

    This flag indicates if a subcommand is going to be executed. A group callback can use this information to figure out if it’s being executed directly or because the execution flow passes onwards to a subcommand. By default it’s None, but it can be the name of the subcommand to execute.

    If chaining is enabled this will be set to `'*'` in case any commands are executed. It is however not possible to figure out which ones. If you require this knowledge you should use a `resultcallback()`.

  `lookup_default`(name, call=True)

    Get the default for a parameter from `default_map`.

    Parameters

      - name – Name of the parameter.

      - call – If the default is a callable, call it. Disable to return the callable instead.

    Changed in version 8.0: Added the `call` parameter.

  `make_formatter`()

    Creates the `HelpFormatter` for the help and usage output.

    To quickly customize the formatter class used without overriding this method, set the `formatter_class` attribute.

    Changed in version 8.0: Added the `formatter_class` attribute.

  `max_content_width`

    The maximum width of formatted content (None implies a sensible default which is 80 for most things).

  property `meta`

    This is a dictionary which is shared with all the contexts that are nested. It exists so that click utilities can store some state here if they need to. It is however the responsibility of that code to manage this dictionary well.

    The keys are supposed to be unique dotted strings. For instance module paths are a good choice for it. What is stored in there is irrelevant for the operation of click. However what is important is that code that places data here adheres to the general semantics of the system.

    Example usage:

        LANG_KEY = f'{__name__}.lang'

        def set_language(value):
            ctx = get_current_context()
            ctx.meta[LANG_KEY] = value

        def get_language():
            return get_current_context().meta.get(LANG_KEY, 'en_US')

    Changelog

    New in version 5.0.

  `obj`

    the user object stored.

  `params`

    Map of parameter names to their parsed values. Parameters with `expose_value=False` are not stored.

  `parent`

    the parent context or None if none exists.

  `protected_args`

    protected arguments. These are arguments that are prepended to args when certain parsing scenarios are encountered but must be never propagated to another arguments. This is used to implement nested parsing.

  `resilient_parsing`

    Indicates if resilient parsing is enabled. In that case Click will do its best to not cause any failures and default values will be ignored. Useful for completion.

  `scope`(cleanup=True)

    This helper method can be used with the context object to promote it to the current thread local (see `get_current_context()`). The default behavior of this is to invoke the cleanup functions which can be disabled by setting cleanup to False. The cleanup functions are typically used for things such as closing file handles.

    If the cleanup is intended the context object can also be directly used as a context manager.

    Example usage:

        async with ctx.scope():
            assert get_current_context() is ctx

    This is equivalent:

        async with ctx:
            assert get_current_context() is ctx

    Changelog

    New in version 5.0.

    Parameters

      cleanup – controls if the cleanup functions should be run or not. The default is to run these functions. In some situations the context only wants to be temporarily pushed in which case this can be disabled. Nested pushes automatically defer the cleanup.

  `set_parameter_source`(name, source)

    Set the source of a parameter. This indicates the location from which the value of the parameter was obtained.

    Parameters

      - name – The name of the parameter.

      - source – A member of `ParameterSource`.

  `show_default`

    Show option default values when formatting help text.

  `terminal_width`

    The width of the terminal (None is autodetection).

  async `to_info_dict`()

    Gather information that could be useful for a tool generating user-facing documentation. This traverses the entire CLI structure.

        async with Context(cli) as ctx:
            info = await ctx.to_info_dict()

    New in version 8.0.

  `token_normalize_func`

    An optional normalization function for tokens. This is options, choices, commands etc.

  async `with_async_resource`(context_manager)

    Register a resource as if it were used in an `async with` statement. The resource will be cleaned up when the context is popped.

    Uses `contextlib.ExitStack.enter_async_context()`. It calls the resource’s `__aenter__()` method and returns the result. When the context is popped, it closes the stack, which calls the resource’s `__aexit__()` method.

    To register a cleanup function for something that isn’t a context manager, use `call_on_close()`. Or use something from `contextlib` to turn it into a context manager first.

        @click.group()
        @click.option("--name")
        @click.pass_context
        def cli(ctx):
            ctx.obj = ctx.with_resource(connect_db(name))

    Parameters

      context_manager – The context manager to enter.

    Returns

      Whatever `context_manager.__enter__()` returns.

    New in version 8.0.

  `with_resource`(context_manager)

    Register a resource as if it were used in a `with` statement. The resource will be cleaned up when the context is popped.

    Uses `contextlib.ExitStack.enter_context()`. It calls the resource’s `__enter__()` method and returns the result. When the context is popped, it closes the stack, which calls the resource’s `__exit__()` method.

    To register a cleanup function for something that isn’t a context manager, use `call_on_close()`. Or use something from `contextlib` to turn it into a context manager first.

        @click.group()
        @click.option("--name")
        @click.pass_context
        def cli(ctx):
            ctx.obj = ctx.with_resource(connect_db(name))

    Parameters

      context_manager – The context manager to enter.

    Returns

      Whatever `context_manager.__enter__()` returns.

    New in version 8.0.

`asyncclick.` `get_current_context`(silent=False)

  Returns the current click context. This can be used as a way to access the current context object from anywhere. This is a more implicit alternative to the `pass_context()` decorator. This function is primarily useful for helpers such as `echo()` which might be interested in changing its behavior based on the current context.

  To push the current context, `Context.scope()` can be used.

  Changelog

  New in version 5.0.

  Parameters

    silent – if set to True the return value is None if no context is available. The default behavior is to raise a `RuntimeError`.

class `asyncclick.core.` `ParameterSource`(value)

  This is an `Enum` that indicates the source of a parameter’s value.

  Use `click.Context.get_parameter_source()` to get the source for a parameter by name.

  Changed in version 8.0: Use `Enum` and drop the `validate` method.

  Changed in version 8.0: Added the `PROMPT` value.

  `COMMANDLINE` = 1

    The value was provided by the command line args.

  `ENVIRONMENT` = 2

    The value was provided with an environment variable.

  `DEFAULT` = 3

    Used the default specified by the parameter.

  `DEFAULT_MAP` = 4

    Used a default provided by `Context.default_map`.

  `PROMPT` = 5

    Used a prompt to confirm a default or provide a value.

## Types

`asyncclick.` `STRING` = STRING

`asyncclick.` `INT` = INT

`asyncclick.` `FLOAT` = FLOAT

`asyncclick.` `BOOL` = BOOL

`asyncclick.` `UUID` = UUID

`asyncclick.` `UNPROCESSED` = UNPROCESSED

class `asyncclick.` `File`(mode='r', encoding=None, errors='strict', lazy=None, atomic=False)

  Declares a parameter to be a file for reading or writing. The file is automatically closed once the context tears down (after the command finished working).

  Files can be opened for reading or writing. The special value `-` indicates stdin or stdout depending on the mode.

  By default, the file is opened for reading text data, but it can also be opened in binary mode or for writing. The encoding parameter can be used to force a specific encoding.

  The lazy flag controls if the file should be opened immediately or upon first IO. The default is to be non-lazy for standard input and output streams as well as files opened for reading, lazy otherwise. When opening a file lazily for reading, it is still opened temporarily for validation, but will not be held open until first IO. lazy is mainly useful when opening for writing to avoid creating the file until it is needed.

  Starting with Click 2.0, files can also be opened atomically in which case all writes go into a separate file in the same folder and upon completion the file will be moved over to the original location. This is useful if a file regularly read by other users is modified.

  See File Arguments for more information.

class `asyncclick.` `Path`(exists=False, file_okay=True, dir_okay=True, writable=False, readable=True, resolve_path=False, allow_dash=False, path_type=None)

  The path type is similar to the `File` type but it performs different checks. First of all, instead of returning an open file handle it returns just the filename. Secondly, it can perform various basic checks about what the file or directory should be.

  Changelog

  Changed in version 6.0: allow_dash was added.

  Parameters

    - exists – if set to true, the file or directory needs to exist for this value to be valid. If this is not required and a file does indeed not exist, then all further checks are silently skipped.

    - file_okay – controls if a file is a possible value.

    - dir_okay – controls if a directory is a possible value.

    - writable – if true, a writable check is performed.

    - readable – if true, a readable check is performed.

    - resolve_path – if this is true, then the path is fully resolved before the value is passed onwards. This means that it’s absolute and symlinks are resolved. It will not expand a tilde-prefix, as this is supposed to be done by the shell only.

    - allow_dash – If this is set to True, a single dash to indicate standard streams is permitted.

    - path_type – optionally a string type that should be used to represent the path. The default is None which means the return value will be either bytes or unicode depending on what makes most sense given the input data Click deals with.

class `asyncclick.` `Choice`(choices, case_sensitive=True)

  The choice type allows a value to be checked against a fixed set of supported values. All of these values have to be strings.

  You should only pass a list or tuple of choices. Other iterables (like generators) may lead to surprising results.

  The resulting value will always be one of the originally passed choices regardless of `case_sensitive` or any `ctx.token_normalize_func` being specified.

  See Choice Options for an example.

  Parameters

    case_sensitive – Set to false to make choices case insensitive. Defaults to true.

class `asyncclick.` `IntRange`(min=None, max=None, min_open=False, max_open=False, clamp=False)

  Restrict an `click.INT` value to a range of accepted values. See Range Options.

  If `min` or `max` are not passed, any value is accepted in that direction. If `min_open` or `max_open` are enabled, the corresponding boundary is not included in the range.

  If `clamp` is enabled, a value outside the range is clamped to the boundary instead of failing.

  Changed in version 8.0: Added the `min_open` and `max_open` parameters.

class `asyncclick.` `FloatRange`(min=None, max=None, min_open=False, max_open=False, clamp=False)

  Restrict a `click.FLOAT` value to a range of accepted values. See Range Options.

  If `min` or `max` are not passed, any value is accepted in that direction. If `min_open` or `max_open` are enabled, the corresponding boundary is not included in the range.

  If `clamp` is enabled, a value outside the range is clamped to the boundary instead of failing. This is not supported if either boundary is marked `open`.

  Changed in version 8.0: Added the `min_open` and `max_open` parameters.

class `asyncclick.` `Tuple`(types)

  The default behavior of Click is to apply a type on a value directly. This works well in most cases, except for when nargs is set to a fixed count and different types should be used for different items. In this case the `Tuple` type can be used. This type can only be used if nargs is set to a fixed number.

  For more information see Tuples as Multi Value Options.

  This can be selected by using a Python tuple literal as a type.

  Parameters

    types – a list of types that should be used for the tuple items.

class `asyncclick.` `ParamType`

  Represents the type of a parameter. Validates and converts values from the command line or Python into the correct type.

  To implement a custom type, subclass and implement at least the following:

  - The `name` class attribute must be set.

  - Calling an instance of the type with `None` must return `None`. This is already implemented by default.

  - `convert()` must convert string values to the correct type.

  - `convert()` must accept values that are already the correct type.

  - It must be able to convert a value if the `ctx` and `param` arguments are `None`. This can occur when converting prompt input.

  `convert`(value, param, ctx)

    Convert the value to the correct type. This is not called if the value is `None` (the missing value).

    This must accept string values from the command line, as well as values that are already the correct type. It may also convert other compatible types.

    The `param` and `ctx` arguments may be `None` in certain situations, such as when converting prompt input.

    If the value cannot be converted, call `fail()` with a descriptive message.

    Parameters

      - value – The value to convert.

      - param – The parameter that is using this type to convert its value. May be `None`.

      - ctx – The current context that arrived at this value. May be `None`.

  `envvar_list_splitter` = None

    if a list of this type is expected and the value is pulled from a string environment variable, this is what splits it up. None means any whitespace. For all parameters the general rule is that whitespace splits them up. The exception are paths and files which are split by `os.path.pathsep` by default (“:” on Unix and “;” on Windows).

  `fail`(message, param=None, ctx=None)

    Helper method to fail with an invalid value message.

  `get_metavar`(param)

    Returns the metavar default for this param if it provides one.

  `get_missing_message`(param)

    Optionally might return extra information about a missing parameter.

    Changelog

    New in version 2.0.

  `name` = None

    the descriptive name of this type

  `shell_complete`(ctx, param, incomplete)

    Return a list of `CompletionItem` objects for the incomplete value. Most types do not provide completions, but some do, and this allows custom types to provide custom completions as well.

    Parameters

      - ctx – Invocation context for this command.

      - param – The parameter that is requesting completion.

      - incomplete – Value being completed. May be empty.

    New in version 8.0.

  `split_envvar_value`(rv)

    Given a value from an environment variable this splits it up into small chunks depending on the defined envvar list splitter.

    If the splitter is set to None, which means that whitespace splits, then leading and trailing whitespace is ignored. Otherwise, leading and trailing splitters usually lead to empty items being included.

  async `to_info_dict`()

    Gather information that could be useful for a tool generating user-facing documentation.

    Use `click.Context.to_info_dict()` to traverse the entire CLI structure.

    New in version 8.0.

## Exceptions

exception `asyncclick.` `ClickException`(message)

  An exception that Click can handle and show to the user.

exception `asyncclick.` `Abort`

  An internal signalling exception that signals Click to abort.

exception `asyncclick.` `UsageError`(message, ctx=None)

  An internal exception that signals a usage error. This typically aborts any further handling.

  Parameters

    - message – the error message to display.

    - ctx – optionally the context that caused this error. Click will fill in the context automatically in some situations.

exception `asyncclick.` `BadParameter`(message, ctx=None, param=None, param_hint=None)

  An exception that formats out a standardized error message for a bad parameter. This is useful when thrown from a callback or type as Click will attach contextual information to it (for instance, which parameter it is).

  Changelog

  New in version 2.0.

  Parameters

    - param – the parameter object that caused this error. This can be left out, and Click will attach this info itself if possible.

    - param_hint – a string that shows up as parameter name. This can be used as alternative to param in cases where custom validation should happen. If it is a string it’s used as such, if it’s a list then each item is quoted and separated.

exception `asyncclick.` `FileError`(filename, hint=None)

  Raised if a file cannot be opened.

exception `asyncclick.` `NoSuchOption`(option_name, message=None, possibilities=None, ctx=None)

  Raised if click attempted to handle an option that does not exist.

  Changelog

  New in version 4.0.

exception `asyncclick.` `BadOptionUsage`(option_name, message, ctx=None)

  Raised if an option is generally supplied but the use of the option was incorrect. This is for instance raised if the number of arguments for an option is not correct.

  Changelog

  New in version 4.0.

  Parameters

    option_name – the name of the option being used incorrectly.

exception `asyncclick.` `BadArgumentUsage`(message, ctx=None)

  Raised if an argument is generally supplied but the use of the argument was incorrect. This is for instance raised if the number of values for an argument is not correct.

  Changelog

  New in version 6.0.

## Formatting

class `asyncclick.` `HelpFormatter`(indent_increment=2, width=None, max_width=None)

  This class helps with formatting text-based help pages. It’s usually just needed for very special internal cases, but it’s also exposed so that developers can write their own fancy outputs.

  At present, it always writes into memory.

  Parameters

    - indent_increment – the additional increment for each level.

    - width – the width for the text. This defaults to the terminal width clamped to a maximum of 78.

  `dedent`()

    Decreases the indentation.

  `getvalue`()

    Returns the buffer contents.

  `indent`()

    Increases the indentation.

  `indentation`()

    A context manager that increases the indentation.

  `section`(name)

    Helpful context manager that writes a paragraph, a heading, and the indents.

    Parameters

      name – the section name that is written as heading.

  `write`(string)

    Writes a unicode string into the internal buffer.

  `write_dl`(rows, col_max=30, col_spacing=2)

    Writes a definition list into the buffer. This is how options and commands are usually formatted.

    Parameters

      - rows – a list of two item tuples for the terms and values.

      - col_max – the maximum width of the first column.

      - col_spacing – the number of spaces between the first and second column.

  `write_heading`(heading)

    Writes a heading into the buffer.

  `write_paragraph`()

    Writes a paragraph into the buffer.

  `write_text`(text)

    Writes re-indented text into the buffer. This rewraps and preserves paragraphs.

  `write_usage`(prog, args='', prefix='Usage: ')

    Writes a usage line into the buffer.

    Parameters

      - prog – the program name.

      - args – whitespace separated list of arguments.

      - prefix – the prefix for the first line.

`asyncclick.` `wrap_text`(text, width=78, initial_indent='', subsequent_indent='', preserve_paragraphs=False)

  A helper function that intelligently wraps text. By default, it assumes that it operates on a single paragraph of text but if the preserve_paragraphs parameter is provided it will intelligently handle paragraphs (defined by two empty lines).

  If paragraphs are handled, a paragraph can be prefixed with an empty line containing the `\b` character (`\x08`) to indicate that no rewrapping should happen in that block.

  Parameters

    - text – the text that should be rewrapped.

    - width – the maximum width for the text.

    - initial_indent – the initial indent that should be placed on the first line as a string.

    - subsequent_indent – the indent string that should be placed on each consecutive line.

    - preserve_paragraphs – if this flag is set then the wrapping will intelligently handle paragraphs.

## Parsing

class `asyncclick.` `OptionParser`(ctx=None)

  The option parser is an internal class that is ultimately used to parse options and arguments. It’s modelled after optparse and brings a similar but vastly simplified API. It should generally not be used directly as the high level Click classes wrap it for you.

  It’s not nearly as extensible as optparse or argparse as it does not implement features that are implemented on a higher level (such as types or defaults).

  Parameters

    ctx – optionally the `Context` where this parser should go with.

  `add_argument`(dest, nargs=1, obj=None)

    Adds a positional argument named dest to the parser.

    The obj can be used to identify the option in the order list that is returned from the parser.

  `add_option`(opts, dest, action=None, nargs=1, const=None, obj=None)

    Adds a new option named dest to the parser. The destination is not inferred (unlike with optparse) and needs to be explicitly provided. Action can be any of `store`, `store_const`, `append`, `appnd_const` or `count`.

    The obj can be used to identify the option in the order list that is returned from the parser.

  `allow_interspersed_args`

    This controls how the parser deals with interspersed arguments. If this is set to False, the parser will stop on the first non-option. Click uses this to implement nested subcommands safely.

  `ctx`

    The `Context` for this parser. This might be None for some advanced use cases.

  `ignore_unknown_options`

    This tells the parser how to deal with unknown options. By default it will error out (which is sensible), but there is a second mode where it will ignore it and continue processing after shifting all the unknown options into the resulting args.

  async `parse_args`(args)

    Parses positional arguments and returns `(values, args, order)` for the parsed options and arguments as well as the leftover arguments if there are any. The order is a list of objects as they appear on the command line. If arguments appear multiple times they will be memorized multiple times as well.

## Shell Completion

See Shell Completion for information about enabling and customizing Click’s shell completion system.

class `asyncclick.shell_completion.` `CompletionItem`(value, type='plain', help=None, **kwargs)

  Represents a completion value and metadata about the value. The default metadata is `type` to indicate special shell handling, and `help` if a shell supports showing a help string next to the value.

  Arbitrary parameters can be passed when creating the object, and accessed using `item.attr`. If an attribute wasn’t passed, accessing it returns `None`.

  Parameters

    - value – The completion suggestion.

    - type – Tells the shell script to provide special completion support for the type. Click uses `"dir"` and `"file"`.

    - help – String shown next to the value if supported.

    - kwargs – Arbitrary metadata. The built-in implementations don’t use this, but custom type completions paired with custom shell support could use it.

class `asyncclick.shell_completion.` `ShellComplete`(cli, ctx_args, prog_name, complete_var)

  Base class for providing shell completion support. A subclass for a given shell will override attributes and methods to implement the completion instructions (`source` and `complete`).

  Parameters

    - cli – Command being called.

    - prog_name – Name of the executable in the shell.

    - complete_var – Name of the environment variable that holds the completion instruction.

  New in version 8.0.

  `name` = None

    Name to register the shell as with `add_completion_class()`. This is used in completion instructions (`{name}_source` and `{name}_complete`).

  `source_template` = None

    Completion script template formatted by `source()`. This must be provided by subclasses.

  property `func_name`

    The name of the shell function defined by the completion script.

  `source_vars`()

    Vars for formatting `source_template`.

    By default this provides `complete_func`, `complete_var`, and `prog_name`.

  `source`()

    Produce the shell script that defines the completion function. By default this `%`-style formats `source_template` with the dict returned by `source_vars()`.

  `get_completion_args`()

    Use the env vars defined by the shell script to return a tuple of `args, incomplete`. This must be implemented by subclasses.

  async `get_completions`(args, incomplete)

    Determine the context and last complete command or parameter from the complete args. Call that object’s `shell_complete` method to get the completions for the incomplete value.

    Parameters

      - args – List of complete args before the incomplete value.

      - incomplete – Value being completed. May be empty.

  `format_completion`(item)

    Format a completion item into the form recognized by the shell script. This must be implemented by subclasses.

    Parameters

      item – Completion item to format.

  async `complete`()

    Produce the completion data to send back to the shell.

    By default this calls `get_completion_args()`, gets the completions, then calls `format_completion()` for each completion.

`asyncclick.shell_completion.` `add_completion_class`(cls, name=None)

  Register a `ShellComplete` subclass under the given name. The name will be provided by the completion instruction environment variable during completion.

  Parameters

    - cls – The completion class that will handle completion for the shell.

    - name – Name to register the class under. Defaults to the class’s `name` attribute.

## Testing

class `asyncclick.testing.` `CliRunner`(charset='utf-8', env=None, echo_stdin=False, mix_stderr=True)

  The CLI runner provides functionality to invoke a Click command line script for unittesting purposes in a isolated environment. This only works in single-threaded systems without any concurrency as it changes the global interpreter state.

  Parameters

    - charset – the character set for the input and output data.

    - env – a dictionary with environment variables for overriding.

    - echo_stdin – if this is set to True, then reading from stdin writes to stdout. This is useful for showing examples in some circumstances. Note that regular prompts will automatically echo the input.

    - mix_stderr – if this is set to False, then stdout and stderr are preserved as independent streams. This is useful for Unix-philosophy apps that have predictable stdout and noisy stderr, such that each may be measured independently

  `get_default_prog_name`(cli)

    Given a command object it will return the default program name for it. The default is the name attribute or `"root"` if not set.

  async `invoke`(cli, args=None, input=None, env=None, catch_exceptions=True, color=False, **extra)

    Invokes a command in an isolated environment. The arguments are forwarded directly to the command line script, the extra keyword arguments are passed to the `main()` function of the command.

    This returns a `Result` object.

    Parameters

      - cli – the command to invoke

      - args – the arguments to invoke. It may be given as an iterable or a string. When given as string it will be interpreted as a Unix shell command. More details at `shlex.split()`.

      - input – the input data for sys.stdin.

      - env – the environment overrides.

      - catch_exceptions – Whether to catch any other exceptions than `SystemExit`.

      - extra – the keyword arguments to pass to `main()`.

      - color – whether the output should contain color codes. The application can still override this explicitly.

    Changed in version 8.0: The result object has the `return_value` attribute with the value returned from the invoked command.

    Changelog

    Changed in version 4.0: Added the `color` parameter.

    Changed in version 3.0: Added the `catch_exceptions` parameter.

    Changed in version 3.0: The result object has the `exc_info` attribute with the traceback if available.

  `isolated_filesystem`()

    A context manager that creates a temporary folder and changes the current working directory to it for isolated filesystem tests.

  `isolation`(input=None, env=None, color=False)

    A context manager that sets up the isolation for invoking of a command line tool. This sets up stdin with the given input data and os.environ with the overrides from the given dictionary. This also rebinds some internals in Click to be mocked (like the prompt functionality).

    This is automatically done in the `invoke()` method.

    Changelog

    New in version 4.0: The `color` parameter was added.

    Parameters

      - input – the input stream to put into sys.stdin.

      - env – the environment overrides as dictionary.

      - color – whether the output should contain color codes. The application can still override this explicitly.

  `make_env`(overrides=None)

    Returns the environment overrides for invoking a script.

class `asyncclick.testing.` `Result`(runner, stdout_bytes, stderr_bytes, return_value, exit_code, exception, exc_info=None)

  Holds the captured result of an invoked CLI script.

  `exc_info`

    The traceback

  `exception`

    The exception that happened if one did.

  `exit_code`

    The exit code as integer.

  property `output`

    The (standard) output as unicode string.

  `return_value`

    The value returned from the invoked command.

    New in version 8.0.

  `runner`

    The runner that created the result

  property `stderr`

    The standard error as unicode string.

  `stderr_bytes`

    The standard error as bytes, or None if not available

  property `stdout`

    The standard output as unicode string.

  `stdout_bytes`

    The standard output as bytes.